<?php
namespace Rethumbify;

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

class Bulk_Process{

    public  $status;

    public function __construct() {

        $this->status = new Status();
    }

    public function is_paused(){
        return $this->status->has_pending_images();
    }
   
    public function reset(){
        wp_cache_delete('rethumbify_total_images');
        wp_cache_delete('rethumbify_image_ids');
        delete_option('rethumbify_completion_email_sent');
        delete_option('rethumbify_start_time');
        $this->status->clear();
    }

    public function is_completed(){
        return $this->status->is_completed();
    }

    public function start() {
        $this->reset();

        $media_library = new Media_Library();
        $image_ids = $media_library->get_image_ids();
        $this->status->mark_as_pending($image_ids);
        update_option('rethumbify_start_time', time());
        return count($image_ids );
    }

    public function process_next() {
        $attachment_id = $this->status->get_next_image();
        if ($attachment_id) {
            $this->process_single_image($attachment_id);
        }
    }

    public function process_single_image($attachment_id) {

        $this->status->update_status($attachment_id, 'processing');

        $rt = new Regenerate_Thumbnails();
        $result = $rt->regenerate_thumbnails($attachment_id);

        if ( is_wp_error( $result ) ) {
            $this->status->update_status($attachment_id, 'failed', $result->get_error_code());
        } else {
            $this->status->update_status($attachment_id, 'completed');
        }

        $this->check_completion();
    }

    private function check_completion(){
        $stats = $this->status->get_stats();
        
        if ($stats['total_pending'] == 0 && 
            $stats['total_processed'] > 0 && 
            $stats['total_processed'] == $stats['total_images']) {
            do_action('rethumbify_regeneration_complete');
        }
    }

    public function is_inprogress(){
        return false;
    }

    public function is_inactive(){
        return true;
    }
    public function pause() {
        return true;
    }

    public function resume() {
        return true;        
    }
    
}