<?php

namespace Rethumbify;

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

class Logger {
    private static $instance = null;
    private $log_file;

    private function __construct() {
        $upload_dir = wp_upload_dir();
        $this->log_file = trailingslashit($upload_dir['basedir']) . 'rethumbify.log';
        
        // Schedule cleanup if not already scheduled
        if (!wp_next_scheduled('rethumbify_cleanup_logs')) {
            wp_schedule_event(time(), 'daily', 'rethumbify_cleanup_logs');
        }
    }

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public function log($message, $level = 'info') {
        $timestamp = current_time('mysql');
        $formatted_message = "[{$timestamp}] [{$level}] {$message}" . PHP_EOL;
        
        if (!file_exists($this->log_file)) {
            wp_mkdir_p(dirname($this->log_file));
        }

        error_log($formatted_message, 3, $this->log_file);
    }

    public function clear_log() {
        if (file_exists($this->log_file)) {
            wp_delete_file($this->log_file);
        }
    }

    public function get_log_contents() {
        if (file_exists($this->log_file)) {
            return file_get_contents($this->log_file);
        }
        return '';
    }

    public static function info($message) {
        self::get_instance()->log($message, 'info');
    }

    public static function error($message) {
        self::get_instance()->log($message, 'error');
    }

    public static function debug($message) {
        self::get_instance()->log($message, 'debug');
    }

    /**
     * Cleanup logs older than one month
     *
     * @return void
     */
    public function cleanup_old_logs() {
        if (!file_exists($this->log_file)) {
            return;
        }

        $logs = file($this->log_file, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        if (!$logs) {
            return;
        }

        $one_month_ago = strtotime('-1 month');
        $new_logs = array();

        foreach ($logs as $log) {
            // Extract timestamp from log entry [YYYY-MM-DD HH:MM:SS]
            if (preg_match('/^\[([\d-]+ [\d:]+)\]/', $log, $matches)) {
                $log_time = strtotime($matches[1]);
                if ($log_time > $one_month_ago) {
                    $new_logs[] = $log;
                }
            }
        }

        // Write filtered logs back to file
        file_put_contents($this->log_file, implode(PHP_EOL, $new_logs) . PHP_EOL);
    }
}