jQuery(document).ready(function ($) {
    var $startButton = $('#rethumbify-start-regeneration');
    var $resumeButton = $('#rethumbify-resume-regeneration');
    var $progressContainer = $('#rethumbify-progress-container');
    var $progressBar = $('#rethumbify-progress');
    var $progressText = $('#rethumbify-progress-text');
    var $pauseButton = $('#rethumbify-pause-regeneration');
    var $spinner = $('.rethumbify-spinner');
    var $settingsForm = $('#rethumbify-settings-form');
    var $bulkRegenerateInfo = $('.rethumbify-bulk-regenerate-info');
    var isRegenerating = false;
    var _processNext = false;

    function handleAjaxError(jqXHR, textStatus, errorThrown, errorMessage) {
        console.error('AJAX error:', textStatus, errorThrown);
        showErrorMessage(errorMessage + ' Please refresh the page to see the current status.');
    }

    function showErrorMessage(message) {
        var $errorMessage = $('<div id="rethumbify-error-message"  class="notice notice-error is-dismissible"><p>' + message + '</p></div>');
        $startButton.after($errorMessage);
    }

    console.log({ rethumbifyData })

    if (rethumbifyData.isRunning) {
        updateProgressState(rethumbifyData.stats);
    }
    else {
        initProgressState();
    }

    $startButton.on('click', function (e) {
        $('#rethumbify-error-message').remove();
        $('.rethumbify-notice').remove();
        $('#rethumbify-failed-images').remove();

        $startButton.hide();
        $resumeButton.hide();
        $progressBar.css('width', '0%');
        $progressText.html('Scanning your media library...');

        $.ajax({
            url: rethumbifyData.ajaxUrl,
            type: 'POST',
            data: {
                action: 'rethumbify_start_regeneration',
                nonce: rethumbifyData.startNonce,
            },
            success: function (response) {
                if (response.success) {
                    updateProgressState(response.data.stats);
                    _processNext = true;
                    processNext();
                } else {
                    $startButton.show();
                    $pauseButton.hide();
                    $spinner.hide();
                    $progressText.hide();
                    $progressContainer.hide();
                    alert(response.data.message);

                }
            },
            error: function (jqXHR, textStatus, errorThrown) {
                handleAjaxError(jqXHR, textStatus, errorThrown, 'An error occurred while starting the regeneration process.');

                $startButton.show();
                $progressContainer.hide();
                $pauseButton.hide();
                $spinner.hide();
            }
        });
    });

    $pauseButton.on('click', function (e) {
        e.preventDefault();
        $('#rethumbify-error-message').remove();
        $resumeButton.hide();
        $startButton.hide();
        _processNext = false;
        $.ajax({
            url: rethumbifyData.ajaxUrl,
            type: 'POST',
            data: {
                action: 'rethumbify_pause_regeneration',
                nonce: rethumbifyData.pauseNonce
            },
            success: function (response) {
                if (response.success) {
                    isRegenerating = false;
                    $progressContainer.hide();
                    $startButton.show();
                    $resumeButton.show();
                    $bulkRegenerateInfo.html($bulkRegenerateInfo.data('paused-info'));
                } else {
                    isRegenerating = !isRegenerating;
                    $resumeButton.show();
                    $startButton.show();
                    $bulkRegenerateInfo.html($bulkRegenerateInfo.data('inprogress-info'));
                    $progressContainer.show();
                    console.error('ReThumbify: ' + (response.data ? response.data.message : 'Unknown error'));
                }
            },
            error: function (xhr, status, error) {
                console.error('Rethumbify:', status, error);
            }
        });
    });

    function initProgressState() {
        if (rethumbifyData.stats.total_images > 0) {
            $progressText.html(rethumbifyData.stats.progress_text);
            updateFailedImagesList(rethumbifyData.stats.failed_images_details);
        }

    }
    function updateProgressState(stats) {
        isRegenerating = true;

        stats.percentage = stats.percentage || 0;
        stats.total_images = stats.total_images || 0;
        stats.total_failed = stats.total_failed || 0;
        stats.total_pending = stats.total_pending || 0;
        stats.total_processed = stats.total_processed || 0;
        stats.progress_text = stats.progress_text || '';
        stats.failed_images_details = stats.failed_images_details || [];

        $startButton.hide();
        $progressContainer.show();
        $progressBar.css('width', stats.percentage + '%');
        $spinner.show();
        $pauseButton.show();
        if (stats.progress_text.length) {
            $progressText.html(stats.progress_text);
        } else {
            $progressText.html('Processed 0 out of ' + stats.total_images + ' images (0%). 0 failed. ' + stats.total_pending + ' pending.');
        }
        $progressText.show();

        if (isRegenerating) {
            $bulkRegenerateInfo.html($bulkRegenerateInfo.data('inprogress-info'));
        } else {
            $bulkRegenerateInfo.html($bulkRegenerateInfo.data('inactive-info'));
        }

        fetchProgress();
    }

    function fetchProgress() {
        if (!isRegenerating) {
            return;
        }

        $.ajax({
            url: ajaxurl,
            type: 'POST',
            data: {
                action: 'rethumbify_get_progress',
                nonce: rethumbifyData.progressNonce
            },
            success: function (response) {
                if (response.success) {
                    renderProgress(response.data);
                    if (response.data.is_completed) {
                        isRegenerating = false;
                        showCompletionMessage(response.data);
                    } else {
                        setTimeout(fetchProgress, 1000);
                    }

                } else {
                    handleAjaxError(null, 'error', null, 'Failed to get progress update.');
                }
            },
            error: function (jqXHR, textStatus, errorThrown) {
                handleAjaxError(jqXHR, textStatus, errorThrown, 'An error occurred while getting the progress update.');
            }
        });
    }

    function renderProgress(data) {
        $progressText.html(data.progress_text);
        $progressBar.css('width', data.percentage + '%');
        if (data.failed_images_details && Object.keys(data.failed_images_details).length > 0) {
            updateFailedImagesList(data.failed_images_details);
        }
    }

    function showCompletionMessage(data) {
        var message = 'All done!';
        var $completionMessage = $('<div class="rethumbify-notice notice notice-success"><p>' + message + '</p></div>');
        $startButton.before($completionMessage);
        $startButton.text('Restart regeneration');
        $startButton.show();
        $resumeButton.hide();
        $progressContainer.hide();
        $spinner.hide();
        $pauseButton.hide();
    }

    function updateFailedImagesList(failedImages) {
        var $failedList = $('#rethumbify-failed-images');
        if (!$failedList.length) {
            $failedList = $('<div id="rethumbify-failed-images" class="rethumbify-failed-images"></div>');
            $('#rethumbify-progress-text').after($failedList);
        }
        $failedList.empty();
        $.each(failedImages, function (attachmentId, details) {
            var $item = $('<div class="rethumbify-failed-image-item"></div>');

            $item.append('<span class="dashicons dashicons-warning" style="color: #eab308;margin-right:10px"></span>');
            $item.append('<span class="rethumbify-failed-image-filename">' + details.filename + '</span>');
            $item.append('<span class="rethumbify-failed-image-message">' + details.message + '</span>');
            $item.append('<a href="#" class="rethumbify-failed-image-action ignore" data-attachment-id="' + attachmentId + '">Ignore</a>');
            $item.append('<a href="#" class="rethumbify-failed-image-action retry" data-attachment-id="' + attachmentId + '">Retry</a>');
            $item.append('<a href="' + details.editLink + '" class="rethumbify-failed-image-action view-details">View Details</a>');

            $failedList.append($item);
        });
        $('.rethumbify-failed-image-action.retry').on('click', function (e) {
            e.preventDefault();
            var attachmentId = $(this).data('attachment-id');
            retryFailedImage(attachmentId);
        });

    }

    function retryFailedImage(attachmentId) {

        var $retryButton = $('.rethumbify-failed-image-action.retry[data-attachment-id="' + attachmentId + '"]');
        var originalText = $retryButton.text();
        $retryButton.text('Retrying...').prop('disabled', true);
        $.ajax({
            url: rethumbifyData.ajaxUrl,
            type: 'POST',
            data: {
                action: 'rethumbify_retry_failed_image',
                nonce: rethumbifyData.retryFailedNonce,
                attachment_id: attachmentId
            },
            success: function (response) {
                if (response.success) {
                    $('.rethumbify-failed-image-item').find('[data-attachment-id="' + attachmentId + '"]').closest('.rethumbify-failed-image-item').remove();
                } else {
                    $retryButton.text(originalText).prop('disabled', false);
                    alert('Error: ' + (response.data ? response.data.message : 'Unknown error'));
                }
            },
            error: function (xhr, status, error) {
                console.error('AJAX error:', status, error);
                $retryButton.text(originalText).prop('disabled', false);
                alert('An error occurred while retrying the failed image.');
            },
            complete: function () {
                $retryButton.text(originalText).prop('disabled', false);
            }
        });
    }

    $settingsForm.on('submit', function (e) {
        e.preventDefault();

        var $submitButton = $settingsForm.find('#submit');
        var $message = $('#rethumbify-settings-message');

        $submitButton.prop('disabled', true);
        $spinner.addClass('is-active');
        $message.removeClass('success error').hide();

        var formData = $settingsForm.serializeArray();
        var settings = {};

        settings['selective_rules'] = [];

        $('.rethumbify-selective-rule').each(function (index) {

            var $rule = $(this);
            var ruleData = {
                condition: $rule.find('.rethumbify-condition-select').val()
            };

            switch (ruleData.condition) {
                case 'upload_date':
                    ruleData.upload_date = $rule.find('.upload_date-field select').val();
                    break;
                case 'post_type':
                    ruleData.post_type = $rule.find('.post_type-field select').val();
                    break;
            }

            settings['selective_rules'].push(ruleData);
        });

        // Convert serialized array to object
        $.each(formData, function (i, field) {
            if (field.name.startsWith('rethumbify_settings[')) {
                var key = field.name.replace('rethumbify_settings[', '').replace(']', '');

                if (key.startsWith('custom_sizes')) {
                    if (!settings.hasOwnProperty('custom_sizes')) {
                        settings['custom_sizes'] = [];
                    }
                    settings['custom_sizes'].push(field.value);
                } else if (key.startsWith('selective_rules')) {
                    var matches = key.match(/selective_rules\[(\d+)\]\[(\w+)\]/);
                    if (matches) {
                        var index = parseInt(matches[1]);
                        var subKey = matches[2];

                        if (!settings.hasOwnProperty('selective_rules')) {
                            settings['selective_rules'] = [];
                        }

                        if (!settings['selective_rules'][index]) {
                            settings['selective_rules'][index] = {};
                        }

                        settings['selective_rules'][index][subKey] = field.value;
                    }
                } else {
                    settings[key] = field.value;
                }
            }
        });

        $.ajax({
            url: rethumbifyData.ajaxUrl,
            type: 'POST',
            data: {
                action: 'rethumbify_save_settings',
                nonce: rethumbifyData.settingsNonce,
                settings: settings
            },
            success: function (response) {
                if (response.success) {
                    $message.text(response.data).removeClass('error').addClass('success').show().delay(3000).fadeOut();
                } else {
                    $message.text(response.data).removeClass('success').addClass('error').show();
                }
            },
            error: function () {
                $message.text('An error occurred. Please try again.').removeClass('success').addClass('error').show();
            },
            complete: function () {
                $submitButton.prop('disabled', false);
                $spinner.removeClass('is-active');
            }
        });
    });

    var $selectiveRules = $('#rethumbify-selective-rules');
    var ruleTemplate = $('#rethumbify-rule-template').html();
    var ruleIndex = $selectiveRules.children().length;
    var availableConditions = ['upload_date', 'post_type'];

    function updateAvailableConditions() {
        var usedConditions = $selectiveRules.find('.rethumbify-condition-select').map(function () {
            return $(this).val();
        }).get();

        $selectiveRules.find('.rethumbify-condition-select').each(function () {
            var $select = $(this);
            var currentValue = $select.val();

            $select.find('option').each(function () {
                var $option = $(this);
                if ($option.val() !== currentValue && usedConditions.includes($option.val())) {
                    $option.prop('disabled', true);
                } else {
                    $option.prop('disabled', false);
                }
            });

            // Show the field for the currently selected condition
            var $rule = $select.closest('.rethumbify-selective-rule');
            $rule.find('.rethumbify-condition-fields > div').hide();
            $rule.find('.' + currentValue + '-field').show();
        });

        if (usedConditions.length === availableConditions.length) {
            $('#rethumbify-add-selective-rule').prop('disabled', true);
        } else {
            $('#rethumbify-add-selective-rule').prop('disabled', false);
        }
    }

    $('#rethumbify-add-selective-rule').on('click', function () {
        console.log('add selective rule');
        var newRule = ruleTemplate.replace(/\{\{index\}\}/g, ruleIndex);
        $selectiveRules.append(newRule);
        var $newRule = $selectiveRules.children().last();
        var $conditionSelect = $newRule.find('.rethumbify-condition-select');

        // Disable already used conditions in the new rule
        var usedConditions = $selectiveRules.find('.rethumbify-condition-select').not($conditionSelect).map(function () {
            return $(this).val();
        }).get();

        $conditionSelect.find('option').each(function () {
            var $option = $(this);
            if (usedConditions.includes($option.val())) {
                $option.prop('disabled', true);
            }
        });

        var selectedCondition = $conditionSelect.val();
        $newRule.find('.' + selectedCondition + '-field').show();
        ruleIndex++;
        updateAvailableConditions();
    });

    $selectiveRules.on('click', '.rethumbify-remove-rule', function () {
        $(this).closest('.rethumbify-selective-rule').remove();
        updateAvailableConditions();
    });

    $selectiveRules.on('change', '.rethumbify-condition-select', function () {
        var $rule = $(this).closest('.rethumbify-selective-rule');
        var condition = $(this).val();
        $rule.find('.rethumbify-condition-fields > div').hide();
        $rule.find('.' + condition + '-field').show();
        updateAvailableConditions();
    });

    // Initialize condition fields visibility and available conditions for existing rules
    $('.rethumbify-condition-select').each(function () {
        $(this).trigger('change');
    });

    if (!($('#rethumbify-add-selective-rule').hasClass('rethumbify-pro-feature') && $('#rethumbify-add-selective-rule').is(':disabled'))) {
        updateAvailableConditions();
    }

    $resumeButton.on('click', function (e) {
        e.preventDefault();
        $('#rethumbify-error-message').remove();
        $('.rethumbify-notice').remove();
        console.log({ rethumbifyData })
        _processNext = false;

        $resumeButton.hide();
        $startButton.hide();
        $progressContainer.show();
        $.ajax({
            url: rethumbifyData.ajaxUrl,
            type: 'POST',
            data: {
                action: 'rethumbify_resume_regeneration',
                nonce: rethumbifyData.resumeNonce
            },
            success: function (response) {
                if (response.success) {
                    updateProgressState(response.data.stats);
                    _processNext = true;
                    processNext();
                } else {
                    isRegenerating = false;
                    $resumeButton.show();
                    $startButton.show();
                    console.log('Error: ' + (response.data ? response.data.message : 'Unknown error'));
                }
            },
            error: function (xhr, status, error) {
                $resumeButton.show();
                $startButton.show();
                console.log('An error occurred while resuming the regeneration process.');
            }
        });
    });

    $(document).on('click', '.rethumbify-failed-image-action.ignore', function (e) {
        e.preventDefault();
        var attachmentId = $(this).data('attachment-id');
        $('.rethumbify-failed-image-item').find('[data-attachment-id="' + attachmentId + '"]').closest('.rethumbify-failed-image-item').remove();
    });

    function processNext() {
        if (!isRegenerating || !_processNext) {
            return;
        }
        jQuery.ajax({
            url: ajaxurl,
            type: 'POST',
            data: {
                action: 'rethumbify_process_next',
                nonce: rethumbifyData.processNextNonce // Add this to your localized data
            },
            success: function (response) {
                if (response.success && isRegenerating && _processNext) {
                    processNext();
                }
            }
        });
    }

});