jQuery(document).ready(function ($) {

$(document).on('click', '.rethumbify-regenerate-button', function(e) {
    e.preventDefault();
    var button = $(this);
    var attachmentId = button.data('attachment-id');
    var originalText = button.text();

    button.text('Regenerating...').prop('disabled', true);
    $.ajax({
        url: rethumbifyMediaData.ajaxUrl,
        type: 'POST',
        data: {
            action: 'rethumbify_regenerate_thumbnails',
            attachment_id: attachmentId,
            nonce: rethumbifyMediaData.regenerateNonce
        },
        success: function(response) {
            if (response.success) {
                alert(response.data.message);
            } else {
                alert('Error: ' + response.data.message);
            }
        },
        error: function() {
            alert('An error occurred while regenerating thumbnails.');
        },
        complete: function() {
            button.text(originalText).prop('disabled', false);
        }
    });
});

// Add modal HTML to the page
$('body').append(`
    <div class="rethumbify-modal-backdrop"></div>
    <div class="rethumbify-modal">
        <div class="rethumbify-modal-header">
            <h2>Image Thumbnails</h2>
            <span class="dashicons dashicons-no-alt rethumbify-modal-close"></span>
        </div>
        <div class="rethumbify-modal-content"></div>
    </div>
`);

$(document).on('click', '.rethumbify-show-subsizes', function(e) {
    e.preventDefault();
    var attachmentId = $(this).data('attachment-id');
    
    // Show modal with loading spinner
    $('.rethumbify-modal-content').html('<div class="rethumbify-loading"><span class="spinner"></span></div>');
    showModal();
    
    $.ajax({
        url: ajaxurl,
        type: 'POST',
        data: {
            action: 'rethumbify_get_subsizes',
            attachment_id: attachmentId,
            nonce: rethumbifyMediaData.subsizesNonce
        },
        success: function(response) {
            if (response.success) {
                var content = `
                    <table class="rethumbify-subsizes-table">
                        <thead>
                            <tr>
                                <th>Size Name</th>
                                <th>Dimensions</th>
                                <th>Action</th>
                            </tr>
                        </thead>
                        <tbody>
                `;
                $.each(response.data, function(name, size) {
                    content += `
                        <tr>
                            <td>${name}</td>
                            <td>${size.width} × ${size.height}</td>
                            <td>
                                <div class="copy-to-clipboard-container">
                                    <input type="text" class="urlfield screen-reader-text" value="${size.url}" readonly>
                                    <button type="button" class="button copy-attachment-url">
                                        Copy URL
                                    </button>
                                    <span class="success hidden" aria-hidden="true">Copied!</span>
                                </div>
                            </td>
                        </tr>
                    `;
                });
                
                content += '</tbody></table>';
                $('.rethumbify-modal-content').html(content);
            }
        }
    });
});

function showModal() {
    $('.rethumbify-modal-backdrop, .rethumbify-modal').show();
}

function hideModal() {
    $('.rethumbify-modal-backdrop, .rethumbify-modal').hide();
}

$('.rethumbify-modal-close, .rethumbify-modal-backdrop').on('click', hideModal);

// Remove the old click handler
$(document).off('click', '.rethumbify-copy-url');

// Add the new clipboard functionality
$(document).on('click', '.copy-attachment-url', function(e) {
    e.preventDefault();
    var $button = $(this);
    var $container = $button.closest('.copy-to-clipboard-container');
    var $input = $container.find('.urlfield');
    var originalText = $button.text();

    // Select the text
    $input.select();
    
    try {
        // Try the new clipboard API first
        navigator.clipboard.writeText($input.val()).then(function() {
            showCopySuccess();
        });
    } catch (err) {
        // Fallback to execCommand
        document.execCommand('copy');
        showCopySuccess();
    }

    function showCopySuccess() {
        $button.text('Copied!');
        setTimeout(function() {
            $button.text(originalText);
        }, 1000);
    }
});

});